{

Name: Slave.spin
Author: John Board
Date: 13:49, 10/08/2012
Purpose: Interface to slave chip
Competition: RCJA State 2012

TODO:

Notes:

}

CON

  START_BIT = 1
  START_CONFIRMED = 2

  PING_ = 3
  PONG = 4

  PIN_DIR = 5
  PIN_VAL = 6

  PING_MIDDLE_MM = 7
  PING_RIGHT_MM = 8

  RAW_QTI_LEFT = 9
  RAW_QTI_MIDDLE = 10
  RAW_QTI_RIGHT = 11

  OPEN_CLAW = 12
  CLOSE_CLAW = 13

  ROBOT_FORWARD = 14
  ROBOT_BACKWARD = 15

  ROBOT_SPIN_LEFT = 16
  ROBOT_SPIN_RIGHT = 17

  MOTOR_LEFT_FORWARD = 18
  MOTOR_LEFT_BACKWARD = 19

  MOTOR_RIGHT_FORWARD = 20
  MOTOR_RIGHT_BACKWARD = 21

  ARM_UP = 22
  ARM_DOWN = 23

  BOTTOM_BUTTON = 24
  TOP_BUTTON = 25

  CPAL_LEFT_COLOR = 26
  CPAL_RIGHT_COLOR = 27

  CPAL_LEFT_RED = 28
  CPAL_LEFT_GREEN = 29
  CPAL_LEFT_BLUE = 30

  CPAL_RIGHT_RED = 31
  CPAL_RIGHT_GREEN = 32
  CPAL_RIGHT_BLUE = 33

  QTI_LEFT_COLOR = 34
  QTI_MIDDLE_COLOR = 35
  QTI_RIGHT_COLOR = 36

  UPDATE_TIME = 37

  WAIT = 38

  TOGGLE_PIN = 39

  REBOOT_ = 40

  ROBOT_HALT = 41

  LCD_CLEAR_SCREEN = 42
  LCD_FIRST_LINE = 43
  LCD_SECOND_LINE = 44
  LCD_THIRD_LINE = 45
  LCD_FORTH_LINE = 46
  LCD_CR = 47
  LCD_CLEAR_FIRST_LINE = 48
  LCD_CLEAR_SECOND_LINE = 49
  LCD_CLEAR_THIRD_LINE = 50
  LCD_CLEAR_FORTH_LINE = 51
  LCD_STR = 52
  LCD_DEC = 53

  PING_MIDDLE_CM = 54
  PING_RIGHT_CM = 55

  SET_LEFT_CPAL_CALIBRATION = 56
  SET_RIGHT_CPAL_CALIBRATION = 57

  SET_QTI_CALIBRATION = 58

OBJ

  Serial: "Serial"
  Profiler[1]: "Profiler"

PUB Start(tx, rx, baud) | temp

  Serial.start(3, 2, 0, 9600)

  'Wait for sync
  repeat until temp == 1
    temp := Serial.RxDec
  Send(1)

  repeat
    Serial.Str(String("Test."))
    Serial.Tx(13)
    waitcnt(clkfreq/100+Cnt)

'-----[Gets and Sets]-----'

PUB SetLeftCPALCalibration(Red, Green, Blue)

  Send(SET_LEFT_CPAL_CALIBRATION)
  Send(Red)
  Send(Green)
  Send(Blue)

PUB SetRightCPALCalibration(Red, Green, Blue)

  Send(SET_RIGHT_CPAL_CALIBRATION)
  Send(Red)
  Send(Green)
  Send(Blue)

PUB SetQTICalibration(Left, Middle, Right)

  Send(SET_QTI_CALIBRATION)
  Send(Left)
  Send(Middle)
  Send(Right)

PUB PinValue(pin, value)

  'Sets HIGH/LOW
  Send(PIN_VAL)
  Send(pin)
  Send(value)

PUB PinDirection(pin, direction)

  'Sets pin Direction
  Send(PIN_DIR)
  Send(pin)
  Send(Direction)

PUB TogglePin(Pin)

  'Toggles state of pin

  Send(TOGGLE_PIN)
  Send(Pin)

PUB GetLeftQTI

  Send(RAW_QTI_LEFT)
  return RecvDec

PUB GetMiddleQTI

  Send(RAW_QTI_MIDDLE)
  return RecvDec

PUB GetRightQTI

  Send(RAW_QTI_RIGHT)
  return RecvDec

PUB GetLeftQTIColor

  Send(QTI_LEFT_COLOR)
  return RecvDec

PUB GetMiddleQTIColor

  Send(QTI_MIDDLE_COLOR)
  return RecvDec

PUB GetRightQTIColor

  Send(QTI_RIGHT_COLOR)
  return RecvDec

PUB GetLeftRed

  Send(CPAL_LEFT_RED)
  return RecvDec

PUB GetLeftGreen

  Send(CPAL_LEFT_GREEN)
  return RecvDec

PUB GetLeftBlue

  Send(CPAL_LEFT_BLUE)
  return RecvDec

PUB GetRightRed

  Send(CPAL_RIGHT_RED)
  return RecvDec

PUB GetRightGreen

  Send(CPAL_RIGHT_GREEN)
  return RecvDec

PUB GetRightBlue

  Send(CPAL_RIGHT_BLUE)
  return RecvDec

PUB GetLeftColor

  Send(CPAL_LEFT_COLOR)
  return RecvDec

PUB GetRightColor

  Send(CPAL_RIGHT_COLOR)
  return RecvDec

PUB GetMiddleMM

  Send(PING_MIDDLE_MM)
  return RecvDec

PUB GetRightMM

  Send(PING_RIGHT_MM)
  return RecvDec

PUB GetMiddleCM

  Send(PING_MIDDLE_CM)
  return RecvDec

PUB GetRightCM

  Send(PING_RIGHT_CM)
  return RecvDec

'-------------------------'

'-----{ Motor }-----'

PUB Forward(speed, time)

  SetForward(Speed)
  Pause(time)
  Halt

PUB SpinLeft(speed, time)

  SetSpinLeft(Speed)
  Pause(time)
  Halt

PUB SpinRight(speed, time)

  SetSpinRight(Speed)
  Pause(time)
  Halt

PUB Backward(speed, time)

  SetBackward(Speed)
  Pause(time)
  Halt

PUB SetForward(speed)

  Send(ROBOT_FORWARD)
  Send(Speed)

PUB SetSpinLeft(speed)

  Send(ROBOT_SPIN_LEFT)
  Send(Speed)

PUB SetSpinRight(speed)

  Send(ROBOT_SPIN_RIGHT)
  Send(Speed)

PUB SetBackward(speed)

  Send(ROBOT_BACKWARD)
  Send(Speed)



PUB Halt

  Send(ROBOT_HALT)

'-----{ LCD }-----'

PUB ClearScreen

  Send(LCD_CLEAR_SCREEN)

PUB FirstLine

  Send(LCD_FIRST_LINE)

PUB SecondLine

  Send(LCD_SECOND_LINE)

PUB ThirdLine

  Send(LCD_THIRD_LINE)

PUB ForthLine

  Send(LCD_FORTH_LINE)

PUB ClearFirstLine

  Send(LCD_CLEAR_FIRST_LINE)

PUB ClearSecondLine

  Send(LCD_CLEAR_SECOND_LINE)

PUB ClearThirdLine

  Send(LCD_CLEAR_THIRD_LINE)

PUB ClearForthLine

  Send(LCD_CLEAR_FORTH_LINE)

PUB CR

  Send(LCD_CR)

PUB Dec(val)

  Send(LCD_DEC)
  Send(val)

PUB Str(val)

  Send(LCD_STR)
  Serial.Str(String("Testing 123"))
  Serial.Tx(13)
  
'-----{ Misc }-----'

PUB GetMainUpdateTime

  Send(UPDATE_TIME)
  return RecvDec

PUB RebootSlave

  Send(REBOOT_)

PUB Send(val)

  Serial.Dec(val)
  Serial.tx(13)

PUB RecvDec

  return Serial.RxDec

PUB RecvStr(pntr)

  Serial.RxStr(pntr)

PUB RecvID(id) | temp

  'Waits for a certain packet to be recieved
  repeat until temp==id
    temp := RecvDec

  return True

PUB RecvIDTimeout(id, time) | temp, cntr

  'Waits for a certain packet to be recieved, timeout of time (in number of trys)
  repeat until temp==id
    cntr++
    if cntr=>time
      return False
    temp := RecvDec

  return True

PUB Ping

  'Returns lag time in ms

  Send(PING_) 'PING'
  Profiler[0].StartTimer

  if not RecvID(PONG)==0  
    return Profiler[0].StopTimerX
  else
    Profiler[0].StopTimer
    return 0

PUB PingTimeout(time)

  'Returns lag time in ms, with try timeout, returns -1 if failure.

  Send(PING) 'PING'
  Profiler[0].StartTimer
  if RecvIDTimeout(PONG, time)
    return Profiler[0].StopTimerX
  else
    return -1

PUB Pause(ms)

  waitcnt((clkfreq/1000)*ms+cnt)
    